package cmd

import (
	"time"

	"github.com/spf13/cobra"
	"github.com/spf13/viper"
	"lab.libreho.st/libre.sh/docker/rc-apps-mgr/internal"
)

var applyCmd = &cobra.Command{
	Use:   "apply",
	Short: "Reconcile Rocket.Chat apps with listed ones.",
	Run: func(cmd *cobra.Command, args []string) {
		apps := internal.FetchAll(viper.GetStringSlice("apps"), false)
		rc := internal.NewRC(viper.GetString("host"), viper.GetString("username"), viper.GetString("password"))
		rc.SetAppDevMode(true)
		time.Sleep(time.Second)
		for _, app := range apps {
			if rc.IsInstalled(app) {
				rc.Update(app)
			} else {
				rc.Install(app)
			}
		}
		if viper.GetBool("uninstall") {
			installed := rc.List()
			for _, id := range installed {
				app := internal.FindApp(apps, func(a internal.App) bool {
					return a.ID == id
				})
				if app == nil {
					rc.Uninstall(id)
				}
			}
		}
		rc.SetAppDevMode(false)
	},
}

func init() {
	rootCmd.AddCommand(applyCmd)
}
