package internal

import (
	"fmt"
	"log"

	"github.com/go-resty/resty/v2"
	"github.com/tidwall/gjson"
)

type RC struct {
	*resty.Client
}

func NewRC(host, username, password string) RC {
	if host == "" || username == "" || password == "" {
		log.Fatal("host, username or password is empty")
	}
	rc := RC{
		resty.New().
			SetBaseURL(host).
			SetHeader("X-User-Id", username).
			SetHeader("X-Auth-Token", password).
			SetHeader("Content-Type", "application/json"),
	}
	rc.Check()
	return rc
}

func (rc RC) Check() {
	resp, err := rc.R().Get("/api/v1/me")
	DieOnError(err)
	DieOnResponseError(resp)
}

func (rc RC) SetAppDevMode(value bool) {
	body := fmt.Sprintf(`{"message":"{\"msg\":\"method\",\"id\":\"30\",\"method\":\"saveSettings\",\"params\":[[{\"_id\":\"Apps_Framework_Development_Mode\",\"value\":%t}]]}"}`, value)
	resp, err := rc.R().SetBody(body).Post("/api/v1/method.call/saveSettings")
	DieOnError(err)
	DieOnResponseError(resp)
}

func (rc RC) IsInstalled(app App) bool {
	resp, err := rc.R().Get(fmt.Sprintf("/api/apps/%s", app.ID))
	DieOnError(err)
	return resp.IsSuccess()
}

func (rc RC) Install(app App) {
	resp, err := rc.R().SetFile("app", LocalPath(app.Filename)).Post("/api/apps")
	DieOnError(err)
	DieOnResponseError(resp)
	log.Printf("%s installed", app.ID)
}

func (rc RC) Update(app App) {
	resp, err := rc.R().SetFile("app", LocalPath(app.Filename)).Post("/api/apps/update")
	DieOnError(err)
	DieOnResponseError(resp)
	log.Printf("%s updated", app.ID)
}

func (rc RC) Uninstall(id string) {
	resp, err := rc.R().Delete(fmt.Sprintf("/api/apps/%s", id))
	DieOnError(err)
	DieOnResponseError(resp)
	log.Printf("%s uninstalled", id)
}

func (rc RC) List() []string {
	resp, err := rc.R().Get("/api/apps")
	DieOnError(err)
	DieOnResponseError(resp)
	list := []string{}
	gjson.Get(string(resp.Body()), "apps.#.id").ForEach(func(key, value gjson.Result) bool {
		list = append(list, value.String())
		return true
	})
	return list
}
